@@ -183,8 +183,13 @@ void MapObjectLabel::syncWithMapObject(const MapRenderer &renderer)
     if (!nameVisible)
         return;
 
+    if (mText != mObject->name()) {
+        mText = mObject->name();
+        update();
+    }
+
     const QFontMetricsF metrics(scene() ? scene()->font() : QApplication::font());
-    QRectF boundingRect = metrics.boundingRect(mObject->name());
+    QRectF boundingRect = metrics.boundingRect(mText);
 
     const qreal margin = Utils::dpiScaled(labelMargin);
     const qreal distance = Utils::dpiScaled(labelDistance);
@@ -251,9 +256,9 @@ void MapObjectLabel::paint(QPainter *painter,
 
     painter->drawRoundedRect(mBoundingRect, 4, 4);
     painter->setPen(Qt::black);
-    painter->drawText(mTextPos + QPointF(1,1), mObject->name());
+    painter->drawText(mTextPos + QPointF(1,1), mText);
     painter->setPen(Qt::white);
-    painter->drawText(mTextPos, mObject->name());
+    painter->drawText(mTextPos, mText);
 }
 
 
@@ -368,6 +373,27 @@ QVariant ObjectSelectionItem::itemChange(GraphicsItemChange change, const QVaria
 void ObjectSelectionItem::changeEvent(const ChangeEvent &event)
 {
     switch (event.type) {
+    case ChangeEvent::DocumentAboutToReload:
+        qDeleteAll(mObjectLabels);
+        qDeleteAll(mObjectOutlines);
+        qDeleteAll(mObjectHoverItems);
+        for (const auto &referenceItems : std::as_const(mReferencesBySourceObject))
+            qDeleteAll(referenceItems);
+
+        mObjectLabels.clear();
+        mObjectOutlines.clear();
+        mObjectHoverItems.clear();
+        mReferencesBySourceObject.clear();
+        mReferencesByTargetObject.clear();
+        break;
+
+    case ChangeEvent::DocumentReloaded:
+        if (objectLabelVisibility() == Preferences::AllObjectLabels)
+            addRemoveObjectLabels();
+
+        if (Preferences::instance()->showObjectReferences())
+            addRemoveObjectReferences();
+        break;
     case ChangeEvent::ObjectsChanged: {
         auto &objectsChange = static_cast<const ObjectsChangeEvent&>(event);
         if (!objectsChange.objects.isEmpty() && (objectsChange.properties & ObjectsChangeEvent::ClassProperty)) {
